#!/usr/bin/env python3
"""Utility script to setup Django and run tests against package"""
import sys
from os.path import dirname, join

from django import VERSION as DjangoVersion, setup
from django.apps import apps
from django.conf import settings
from django.core.management import execute_from_command_line

try:
    import improved_user  # noqa: F401 pylint: disable=unused-import
except ImportError:
    print(
        'Could not load improved_user!\n'
        'Try running `./setup.py develop` before `./runtests.py`\n'
        'or run `./setup.py test` for an all in one solution',
    )
    exit(-1)


def configure_django():
    """Configure Django before tests"""
    middleware = [
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ]

    if DjangoVersion >= (1, 10):
        middleware_var_name = 'MIDDLEWARE'
    else:
        middleware_var_name = 'MIDDLEWARE_CLASSES'

    middleware_kwargs = {
        middleware_var_name: middleware,
    }

    settings.configure(
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
                'NAME': ':memory:',
            },
        },
        INSTALLED_APPS=[
            'django.contrib.admin',
            'django.contrib.auth',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'improved_user.apps.ImprovedUserConfig',
        ],
        SITE_ID=1,
        AUTH_USER_MODEL='improved_user.User',
        FIXTURE_DIRS=(join(dirname(__file__), 'tests', 'fixtures'),),
        TEMPLATES=[{
            'BACKEND': 'django.template.backends.django.DjangoTemplates',
            'DIRS': [],
            'APP_DIRS': True,
            'OPTIONS': {
                'context_processors': [
                    'django.contrib.auth.context_processors.auth',
                    'django.contrib.messages.context_processors.messages',
                ],
            },
        }],
        # TODO: when Dj1.8 dropped, use MIDDLEWARE directly
        **middleware_kwargs  # noqa: C815
    )
    setup()


def run_test_suite(*args):
    """Run the test suite"""
    test_args = list(args) or []
    execute_from_command_line(['manage.py', 'test'] + test_args)


def check_missing_migrations():
    """Check that user model and migration files are in sync"""
    from django.db.migrations.autodetector import MigrationAutodetector
    from django.db.migrations.loader import MigrationLoader
    from django.db.migrations.state import ProjectState
    try:
        from django.db.migrations.questioner import (
            NonInteractiveMigrationQuestioner as Questioner,
        )
    except ImportError:
        # TODO: remove this once Dj1.8 dropped
        from django.db.migrations.questioner import (
            InteractiveMigrationQuestioner as Questioner,
        )

    loader = MigrationLoader(None, ignore_no_migrations=True)
    conflicts = loader.detect_conflicts()
    if conflicts:
        raise Exception(
            'Migration conflicts detected. Please fix your migrations.')
    questioner = Questioner(dry_run=True, specified_apps=None)
    autodetector = MigrationAutodetector(
        loader.project_state(),
        ProjectState.from_apps(apps),
        questioner,
    )
    changes = autodetector.changes(
        graph=loader.graph,
        trim_to_apps=None,
        convert_apps=None,
        migration_name=None,
    )
    if changes:
        raise Exception(
            'Migration changes detected. '
            'Please update or add to the migration file as appropriate')
    print('Migration-checker detected no problems.')


if __name__ == '__main__':
    configure_django()
    check_missing_migrations()
    run_test_suite(*sys.argv[1:])



"""Test Signal Handling"""
from django.db.models.signals import post_save
from django.test import TestCase

from improved_user.models import User


class TestCreateSuperUserSignals(TestCase):
    """Simple test case for ticket #20541"""

    # pylint: disable=unused-argument
    def post_save_listener(self, *args, **kwargs):
        """Utility function to note when signal sent"""
        self.signals_count += 1
    # pylint: enable=unused-argument

    def setUp(self):
        """Connect function above to postsave User model signal"""
        self.signals_count = 0
        post_save.connect(self.post_save_listener, sender=User)

    def tearDown(self):
        """Connect utility function from postsave"""
        post_save.disconnect(self.post_save_listener, sender=User)

    def test_create_user(self):
        """Test User Creation"""
        User.objects.create_user('mail@example.com')
        self.assertEqual(self.signals_count, 1)

    def test_create_superuser(self):
        """Test Super User Creation"""
        User.objects.create_superuser('mail@example.com', 'password')
        self.assertEqual(self.signals_count, 1)



"""The Improved User Model

Mixin classes used to create this class may be found in mixins.py

The UserManager is found in managers.py

"""
from .mixins import AbstractUser


# pylint: disable=too-many-ancestors
class User(AbstractUser):
    """The Improved User Model is intended to be used out-of-the-box.

    Do **not** import this model directly: use
    :py:func:`~django.contrib.auth.get_user_model`.
    """



"""Factories to make testing with Improved User easier"""
from django.contrib.auth import get_user_model

try:
    from factory import Faker, PostGenerationMethodCall
    from factory.django import DjangoModelFactory
except (ImportError, ModuleNotFoundError):  # pragma: no cover
    raise Exception(
        "Please install factory_boy to use Improved User's UserFactory.\n"
        'pip install factory_boy==2.9.2')

User = get_user_model()  # pylint: disable=invalid-name


# pylint: disable=too-few-public-methods
class UserFactory(DjangoModelFactory):
    """Factory Boy factory for Improved User

    Generates a user with a default password of ``password!``.
    The user is active, but is not staff or a superuser.
    Any value can be overridden by passing in a value, as shown below.

    .. code:: python

        UserFactory(
            password='mys3cr3tp4ssw0rd!',
            is_superuser=True,
        )
    """
    class Meta:
        """Configuration Options"""
        model = User

    email = Faker('email')
    password = PostGenerationMethodCall('set_password', 'password!')
    full_name = Faker('name')
    short_name = Faker('first_name')
    is_active = True
    is_staff = False
    is_superuser = False
# pylint: enable=too-few-public-methods



"""Test User model management commands"""
import builtins
from io import StringIO

from django import VERSION as DjangoVersion
from django.contrib.auth.management.commands import createsuperuser
from django.core.management import call_command
from django.core.management.base import CommandError
from django.test import TestCase, override_settings

from improved_user.models import User


# pylint: disable=missing-docstring,too-few-public-methods
def mock_inputs(inputs):
    """
    Decorator to temporarily replace input/getpass to allow interactive
    createsuperuser.
    """
    def inner(test_func):
        def wrapped(*args):
            class MockGetPass:
                # pylint: disable=unused-argument
                @staticmethod
                def getpass(prompt=b'Password: ', stream=None):
                    if callable(inputs['password']):
                        return inputs['password']()
                    return inputs['password']
                # pylint: enable=unused-argument

            def mock_input(prompt):
                assert '__proxy__' not in prompt
                response = ''
                for key, val in inputs.items():
                    if key in prompt.lower():
                        if callable(val):
                            response = val()
                        else:
                            response = val
                        break
                return response

            old_getpass = createsuperuser.getpass
            old_input = builtins.input
            createsuperuser.getpass = MockGetPass
            builtins.input = mock_input
            try:
                test_func(*args)
            finally:
                createsuperuser.getpass = old_getpass
                builtins.input = old_input
        return wrapped
    return inner


class MockTTY:
    """
    A fake stdin object that pretends to be a TTY to be used in conjunction
    with mock_inputs.
    """
    def isatty(self):  # pylint: disable=no-self-use
        return True
# pylint: enable=missing-docstring,too-few-public-methods


@override_settings(
    AUTH_PASSWORD_VALIDATORS=[{
        'NAME':
            'django.contrib.auth.password_validation.'
            'NumericPasswordValidator'}])
class CreatesuperuserManagementCommandTestCase(TestCase):
    """Test createsuperuser management command"""

    def test_basic_usage(self):
        """Check the operation of the createsuperuser management command"""
        # We can use the management command to create a superuser
        new_io = StringIO()
        call_command(
            'createsuperuser',
            interactive=False,
            email='joe@somewhere.org',
            short_name='Joe',
            full_name='Joe Smith',
            stdout=new_io,
        )
        command_output = new_io.getvalue().strip()
        self.assertEqual(command_output, 'Superuser created successfully.')
        user = User.objects.get(email='joe@somewhere.org')
        self.assertEqual(user.short_name, 'Joe')

        # created password should be unusable
        self.assertFalse(user.has_usable_password())

    def test_no_email_argument(self):
        """Ensure that email is required"""
        new_io = StringIO()
        with self.assertRaisesMessage(CommandError,
                                      'You must use --email with --noinput.'):
            call_command(
                'createsuperuser',
                interactive=False,
                short_name='Joe',
                full_name='Joe Smith',
                stdout=new_io)

    def test_invalid_username(self):
        """Creation fails if the username fails validation."""
        email_field = User._meta.get_field('email')
        new_io = StringIO()
        invalid_username = ('x' * email_field.max_length) + 'y'

        expected_error = (
            'Enter a valid email address.; '
            'Ensure this value has at most %d characters (it has %d).'
            % (email_field.max_length, len(invalid_username))
        )

        with self.assertRaisesMessage(CommandError, expected_error):
            call_command(
                'createsuperuser',
                interactive=False,
                email=invalid_username,
                short_name='Joe',
                full_name='Joe Smith',
                stdout=new_io)

    def test_password_validation(self):
        """Creation should fail if the password fails validation."""
        new_io = StringIO()

        index = [0]

        # Returns '1234567890' the first two times it is called, then
        # 'password' subsequently.
        def bad_then_good_password():
            """Simulate user input on terminal"""
            index[0] += 1
            if index[0] <= 2:
                return '1234567890'
            return 'password'

        @mock_inputs({
            'password': bad_then_good_password,
        })
        def test(self):
            """The actual test function; run with multiple inputs"""
            call_command(
                'createsuperuser',
                email='hello@jambonsw.com',
                full_name='Joe Smith',
                interactive=True,
                short_name='Joe',
                stderr=new_io,
                stdin=MockTTY(),
                stdout=new_io,
            )
            if DjangoVersion >= (1, 9):
                expected_out = ('This password is entirely numeric.\n'
                                'Superuser created successfully.')
            else:
                expected_out = 'Superuser created successfully.'
            self.assertEqual(new_io.getvalue().strip(), expected_out)

        test(self)



"""Test Improved User against Django's default backend"""
# pylint: disable=protected-access
from unittest.mock import patch

from django import VERSION as DjangoVersion
from django.contrib.auth import authenticate
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.hashers import MD5PasswordHasher
from django.contrib.auth.models import AnonymousUser, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, modify_settings, override_settings

from improved_user.models import User


class CountingMD5PasswordHasher(MD5PasswordHasher):
    """Hasher that counts how many times it computes a hash."""

    calls = 0

    def encode(self, *args, **kwargs):  # pylint: disable=arguments-differ
        """Add counter and call superclass to actually hash"""
        type(self).calls += 1
        return super().encode(*args, **kwargs)


class ImprovedUserModelBackendTest(TestCase):
    """
    Tests for the ModelBackend using the Improved User model.

    Looks for problems that may arise because auth.User
    is missing from db.

    The specific problem is queries on .filter(groups__user) et al,
    which makes an implicit assumption that the user model is called
    'User'. In production, the auth.User table won't exist, so the
    requested join won't exist either; in testing, the auth.User *does*
    exist, and so does the join. However, the join table won't contain
    any useful data; for testing, we check that the data we expect
    actually does exist.
    """

    UserModel = User
    backend = 'django.contrib.auth.backends.ModelBackend'

    def create_users(self):
        """Test utility to create users"""
        self.user = User._default_manager.create_user(
            email='test@example.com',
            password='test',
        )
        self.superuser = User._default_manager.create_superuser(
            email='test2@example.com',
            password='test',
        )

    def test_authenticate(self):
        """Additional test to ensure authentication"""
        authenticated_user = authenticate(
            email='test@example.com', password='test')
        self.assertEqual(self.user, authenticated_user)

    # Code below this point is from Django's BaseModelBackend superclass

    def setUp(self):
        """Add the backend to current settings"""
        self.patched_settings = modify_settings(
            AUTHENTICATION_BACKENDS={'append': self.backend},
        )
        self.patched_settings.enable()
        self.create_users()

    def tearDown(self):
        """Remove patched settings and flush cache"""
        self.patched_settings.disable()
        # The custom_perms test messes with ContentTypes, which will
        # be cached; flush the cache to ensure there are no side effects
        # Refs #14975, #14925
        ContentType.objects.clear_cache()

    def test_has_perm(self):
        """Refresh the user model to test permissions"""
        user = self.UserModel._default_manager.get(pk=self.user.pk)
        self.assertIs(user.has_perm('auth.test'), False)

        user.is_staff = True
        user.save()
        self.assertIs(user.has_perm('auth.test'), False)

        user.is_superuser = True
        user.save()
        self.assertIs(user.has_perm('auth.test'), True)

        user.is_staff = True
        user.is_superuser = True
        user.is_active = False
        user.save()
        self.assertIs(user.has_perm('auth.test'), False)

    def test_custom_perms(self):
        """Check interactions with custom permissions and groups"""
        user = self.UserModel._default_manager.get(pk=self.user.pk)
        content_type = ContentType.objects.get_for_model(Group)
        perm = Permission.objects.create(
            name='test', content_type=content_type, codename='test')
        user.user_permissions.add(perm)

        # reloading user to purge the _perm_cache
        user = self.UserModel._default_manager.get(pk=self.user.pk)
        self.assertEqual(user.get_all_permissions(), {'auth.test'})
        self.assertEqual(user.get_group_permissions(), set())
        self.assertIs(user.has_module_perms('Group'), False)
        self.assertIs(user.has_module_perms('auth'), True)

        perm = Permission.objects.create(
            name='test2', content_type=content_type, codename='test2')
        user.user_permissions.add(perm)
        perm = Permission.objects.create(
            name='test3', content_type=content_type, codename='test3')
        user.user_permissions.add(perm)
        user = self.UserModel._default_manager.get(pk=self.user.pk)
        self.assertEqual(
            user.get_all_permissions(),
            {'auth.test2', 'auth.test', 'auth.test3'})
        self.assertIs(user.has_perm('test'), False)
        self.assertIs(user.has_perm('auth.test'), True)
        self.assertIs(user.has_perms(['auth.test2', 'auth.test3']), True)

        perm = Permission.objects.create(
            name='test_group',
            content_type=content_type,
            codename='test_group')
        group = Group.objects.create(name='test_group')
        group.permissions.add(perm)
        user.groups.add(group)
        user = self.UserModel._default_manager.get(pk=self.user.pk)
        exp = {'auth.test2', 'auth.test', 'auth.test3', 'auth.test_group'}
        self.assertEqual(user.get_all_permissions(), exp)
        self.assertEqual(user.get_group_permissions(), {'auth.test_group'})
        self.assertIs(user.has_perms(['auth.test3', 'auth.test_group']), True)

        user = AnonymousUser()
        self.assertIs(user.has_perm('test'), False)
        self.assertIs(user.has_perms(['auth.test2', 'auth.test3']), False)

    def test_has_no_object_perm(self):
        """Verify proper return of perm object check

        Regression test for #12462
        https://code.djangoproject.com/ticket/12462

        """
        user = self.UserModel._default_manager.get(pk=self.user.pk)
        content_type = ContentType.objects.get_for_model(Group)
        perm = Permission.objects.create(
            name='test', content_type=content_type, codename='test')
        user.user_permissions.add(perm)

        self.assertIs(user.has_perm('auth.test', 'object'), False)
        self.assertEqual(user.get_all_permissions('object'), set())
        self.assertIs(user.has_perm('auth.test'), True)
        self.assertEqual(user.get_all_permissions(), {'auth.test'})

    def test_anonymous_has_no_permissions(self):
        """Anonymous users shouldn't have permissions in ModelBackend

        #17903 -- Anonymous users shouldn't have permissions in
        ModelBackend.get_(all|user|group)_permissions().

        https://code.djangoproject.com/ticket/17903
        """
        backend = ModelBackend()

        user = self.UserModel._default_manager.get(pk=self.user.pk)
        content_type = ContentType.objects.get_for_model(Group)
        user_perm = Permission.objects.create(
            name='test', content_type=content_type, codename='test_user')
        group_perm = Permission.objects.create(
            name='test2', content_type=content_type, codename='test_group')
        user.user_permissions.add(user_perm)

        group = Group.objects.create(name='test_group')
        user.groups.add(group)
        group.permissions.add(group_perm)

        self.assertEqual(
            backend.get_all_permissions(user),
            {'auth.test_user', 'auth.test_group'})
        self.assertEqual(
            backend.get_user_permissions(user),
            {'auth.test_user', 'auth.test_group'})
        self.assertEqual(
            backend.get_group_permissions(user),
            {'auth.test_group'})

        # In Django 1.10, is_anonymous became a property.
        if DjangoVersion >= (1, 10):
            is_anon_mock = True
        else:
            is_anon_mock = lambda s: True  # noqa: E731
        with patch.object(self.UserModel, 'is_anonymous', is_anon_mock):
            self.assertEqual(backend.get_all_permissions(user), set())
            self.assertEqual(backend.get_user_permissions(user), set())
            self.assertEqual(backend.get_group_permissions(user), set())

    def test_inactive_has_no_permissions(self):
        """Inactive users shouldn't have permissions in ModelBackend

        #17903 -- Inactive users shouldn't have permissions in
        ModelBackend.get_(all|user|group)_permissions().

        https://code.djangoproject.com/ticket/17903
        """
        backend = ModelBackend()

        user = self.UserModel._default_manager.get(pk=self.user.pk)
        content_type = ContentType.objects.get_for_model(Group)
        user_perm = Permission.objects.create(
            name='test', content_type=content_type, codename='test_user')
        group_perm = Permission.objects.create(
            name='test2', content_type=content_type, codename='test_group')
        user.user_permissions.add(user_perm)

        group = Group.objects.create(name='test_group')
        user.groups.add(group)
        group.permissions.add(group_perm)

        self.assertEqual(
            backend.get_all_permissions(user),
            {'auth.test_user', 'auth.test_group'})
        self.assertEqual(
            backend.get_user_permissions(user),
            {'auth.test_user', 'auth.test_group'})
        self.assertEqual(
            backend.get_group_permissions(user),
            {'auth.test_group'})

        user.is_active = False
        user.save()

        self.assertEqual(backend.get_all_permissions(user), set())
        self.assertEqual(backend.get_user_permissions(user), set())
        self.assertEqual(backend.get_group_permissions(user), set())

    def test_get_all_superuser_permissions(self):
        """A superuser has all permissions.

        Refs #14795.
        https://code.djangoproject.com/ticket/14795

        """
        user = self.UserModel._default_manager.get(pk=self.superuser.pk)
        self.assertEqual(
            len(user.get_all_permissions()),
            len(Permission.objects.all()))

    @override_settings(
        PASSWORD_HASHERS=[
            'tests.test_auth_backends.CountingMD5PasswordHasher'])
    def test_authentication_timing(self):
        """Hasher is run once regardless of whether the user exists.

        Refs #20760.
        https://code.djangoproject.com/ticket/20760

        """
        # Re-set the password, because this tests overrides PASSWORD_HASHERS
        self.user.set_password('test')
        self.user.save()

        CountingMD5PasswordHasher.calls = 0
        username = getattr(self.user, self.UserModel.USERNAME_FIELD)
        authenticate(username=username, password='test')
        self.assertEqual(CountingMD5PasswordHasher.calls, 1)

        CountingMD5PasswordHasher.calls = 0
        authenticate(username='no_such_user', password='test')
        self.assertEqual(CountingMD5PasswordHasher.calls, 1)



"""Test Admin interface provided by Improved User"""
import os
import re

from django import VERSION as DjangoVersion
from django.contrib.admin.models import LogEntry
from django.contrib.auth import SESSION_KEY
from django.test import TestCase, override_settings
from django.test.utils import patch_logger
from django.utils.encoding import force_text

from improved_user.admin import UserAdmin
from improved_user.forms import UserChangeForm, UserCreationForm
from improved_user.models import User

# TODO: remove conditional import when Dj 1.8 dropped
# pylint: disable=ungrouped-imports
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
# pylint: enable=ungrouped-imports


# Redirect in test_user_change_password will fail if session auth hash
# isn't updated after password change (#21649)
@override_settings(ROOT_URLCONF='tests.urls')
class UserAdminTests(TestCase):
    """Based off django.tests.auth_tests.test_views.ChangelistTests"""

    @classmethod
    def setUpTestData(cls):
        """Called by TestCase during initialization; creates users"""
        cls.user1 = User.objects.create_user(
            email='testclient@example.com',
            password='password',
        )
        cls.user2 = User.objects.create_user(
            email='staffmember@example.com',
            password='password',
        )

    def setUp(self):
        """Make user1 a superuser before logging in."""
        User.objects\
            .filter(email='testclient@example.com')\
            .update(is_staff=True, is_superuser=True)
        self.login()
        self.admin = User.objects.get(pk=self.user1.pk)

    def login(self, username='testclient@example.com', password='password'):
        """Helper function to login the user (specified or default)"""
        response = self.client.post('/login/', {
            'username': username,
            'password': password,
        })
        self.assertIn(SESSION_KEY, self.client.session)
        return response

    def logout(self):
        """Helper function to logout the user"""
        response = self.client.get('/admin/logout/')
        self.assertEqual(response.status_code, 200)
        self.assertNotIn(SESSION_KEY, self.client.session)

    def get_user_data(self, user):  # pylint: disable=no-self-use
        """Generate dictionary of values to compare against"""
        return {
            'email': user.email,
            'password': user.password,
            'is_active': user.is_active,
            'is_staff': user.is_staff,
            'is_superuser': user.is_superuser,
            'last_login_0': user.last_login.strftime('%Y-%m-%d'),
            'last_login_1': user.last_login.strftime('%H:%M:%S'),
            'initial-last_login_0': user.last_login.strftime('%Y-%m-%d'),
            'initial-last_login_1': user.last_login.strftime('%H:%M:%S'),
            'date_joined_0': user.date_joined.strftime('%Y-%m-%d'),
            'date_joined_1': user.date_joined.strftime('%H:%M:%S'),
            'initial-date_joined_0': user.date_joined.strftime('%Y-%m-%d'),
            'initial-date_joined_1': user.date_joined.strftime('%H:%M:%S'),
            'full_name': user.full_name,
            'short_name': user.short_name,
        }

    def test_display_fields(self):
        """Test that admin shows all user fields"""
        excluded_model_fields = ['id', 'logentry']
        model_fields = set(
            field.name for field in User._meta.get_fields()
            if field.name not in excluded_model_fields
        )
        admin_fieldset_fields = set(
            fieldname
            for name, fieldset in UserAdmin.fieldsets
            for fieldname in fieldset['fields']
        )
        self.assertEqual(model_fields, admin_fieldset_fields)

    def test_add_has_required_fields(self):
        """Test all required fields in Admin Add view"""
        excluded_model_fields = ['date_joined', 'password']
        required_model_fields = [
            field.name
            for field in User._meta.get_fields()
            if (field.name not in excluded_model_fields
                and hasattr(field, 'null') and field.null is False
                and hasattr(field, 'blank') and field.blank is False)
        ]
        extra_form_fields = [
            field_name for field_name in list(UserCreationForm.declared_fields)
        ]
        admin_add_fields = [
            fieldname
            for name, fieldset in UserAdmin.add_fieldsets
            for fieldname in fieldset['fields']
        ]
        for field in required_model_fields+extra_form_fields:
            with self.subTest(field=field):
                self.assertIn(field, admin_add_fields)

    def test_correct_forms_used(self):
        """Test that UserAdmin uses the right forms"""
        self.assertIs(UserAdmin.add_form, UserCreationForm)
        self.assertIs(UserAdmin.form, UserChangeForm)

    def test_user_add(self):
        """Ensure the admin add view works correctly"""
        # we can get the form view
        get_response = self.client.get(
            reverse('auth_test_admin:improved_user_user_add'))
        self.assertEqual(get_response.status_code, 200)
        # we can create new users in the form view
        post_response = self.client.post(
            reverse('auth_test_admin:improved_user_user_add'),
            {
                'email': 'newuser@example.com',
                'password1': 'passw0rd1!',
                'password2': 'passw0rd1!',
            },
            follow=True,
        )
        self.assertEqual(post_response.status_code, 200)
        self.assertTrue(
            User.objects.filter(email='newuser@example.com').exists())
        new_user = User.objects.get(email='newuser@example.com')
        self.assertTrue(new_user.check_password('passw0rd1!'))

    def test_user_change_email(self):
        """Test that user can change email in Admin"""
        data = self.get_user_data(self.admin)
        data['email'] = 'new_' + data['email']
        response = self.client.post(
            reverse(
                'auth_test_admin:improved_user_user_change',
                args=(self.admin.pk,),
            ),
            data,
        )
        self.assertRedirects(
            response,
            reverse('auth_test_admin:improved_user_user_changelist'))
        row = LogEntry.objects.latest('id')
        if DjangoVersion >= (1, 9):
            self.assertEqual(row.get_change_message(), 'Changed email.')
        else:
            self.assertEqual(row.change_message, 'Changed email.')

    def test_user_not_change(self):
        """Test that message is raised when form submitted unchanged"""
        response = self.client.post(
            reverse(
                'auth_test_admin:improved_user_user_change',
                args=(self.admin.pk,),
            ),
            self.get_user_data(self.admin),
        )
        self.assertRedirects(
            response,
            reverse('auth_test_admin:improved_user_user_changelist'))
        row = LogEntry.objects.latest('id')
        if DjangoVersion >= (1, 9):
            self.assertEqual(row.get_change_message(), 'No fields changed.')
        else:
            self.assertEqual(row.change_message, 'No fields changed.')

    def test_user_change_password(self):
        """Test that URL to change password form is correct"""
        user_change_url = reverse(
            'auth_test_admin:improved_user_user_change', args=(self.admin.pk,))
        password_change_url = reverse(
            'auth_test_admin:auth_user_password_change',
            args=(self.admin.pk,))

        response = self.client.get(user_change_url)
        # Test the link inside password field help_text.
        rel_link = re.search(
            r'you can change the password using '
            r'<a href="([^"]*)">this form</a>',
            force_text(response.content),
        ).groups()[0]
        self.assertEqual(
            os.path.normpath(user_change_url + rel_link),
            os.path.normpath(password_change_url),
        )

        response = self.client.post(
            password_change_url,
            {
                'password1': 'password1',
                'password2': 'password1',
            },
        )
        self.assertRedirects(response, user_change_url)
        row = LogEntry.objects.latest('id')
        if DjangoVersion >= (1, 9):
            self.assertEqual(row.get_change_message(), 'Changed password.')
        else:
            self.assertEqual(row.change_message, 'Changed password.')
        self.logout()
        self.login(password='password1')

    def test_user_change_password_subclass_path(self):
        """Test subclasses can override password URL"""
        class CustomChangeForm(UserChangeForm):
            """Subclass of UserChangeForm; uses rel_password_url"""
            rel_password_url = 'moOps'

        form = CustomChangeForm()
        self.assertEqual(form.rel_password_url, 'moOps')
        rel_link = re.search(
            r'you can change the password using '
            r'<a href="([^"]*)">this form</a>',
            form.fields['password'].help_text,
        ).groups()[0]
        self.assertEqual(rel_link, 'moOps')

    def test_user_change_different_user_password(self):
        """Test that administrator can update other Users' passwords"""
        user = User.objects.get(email='staffmember@example.com')
        response = self.client.post(
            reverse(
                'auth_test_admin:auth_user_password_change',
                args=(user.pk,),
            ),
            {
                'password1': 'password1',
                'password2': 'password1',
            },
        )
        self.assertRedirects(
            response,
            reverse(
                'auth_test_admin:improved_user_user_change',
                args=(user.pk,)))
        row = LogEntry.objects.latest('id')
        self.assertEqual(row.user_id, self.admin.pk)
        self.assertEqual(row.object_id, str(user.pk))
        if DjangoVersion >= (1, 9):
            self.assertEqual(row.get_change_message(), 'Changed password.')
        else:
            self.assertEqual(row.change_message, 'Changed password.')

    def test_changelist_disallows_password_lookups(self):
        """Users shouldn't be allowed to guess password

        Checks against repeated password__startswith queries
        https://code.djangoproject.com/ticket/20078

        """
        # A lookup that tries to filter on password isn't OK
        with patch_logger(
            'django.security.DisallowedModelAdminLookup', 'error',
        ) as logger_calls:
            response = self.client.get(
                reverse('auth_test_admin:improved_user_user_changelist')
                + '?password__startswith=sha1$')
            self.assertEqual(response.status_code, 400)
            self.assertEqual(len(logger_calls), 1)



"""Mixin Models to allow for easy creation of User models"""
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from .managers import UserManager


class DjangoIntegrationMixin(models.Model):
    """Mixin provides fields for Django integration to work correctly

    Provides permissions for Django Admin integration, as well as date
    field used by authentication code.
    """
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_(
            'Designates whether the user can log into the admin site.'))
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as '
            'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    class Meta:
        abstract = True


class FullNameMixin(models.Model):
    """A mixin to provide an optional full name field"""
    full_name = models.CharField(_('full name'), max_length=200, blank=True)

    class Meta:
        abstract = True

    def get_full_name(self):
        """Returns the full name of the user."""
        return self.full_name


class ShortNameMixin(models.Model):
    """A mixin to provide an optional short name field"""
    short_name = models.CharField(_('short name'), max_length=50, blank=True)

    class Meta:
        abstract = True

    def get_short_name(self):
        """Returns the short name for the user."""
        return self.short_name


class EmailAuthMixin(models.Model):
    """A mixin to use email as the username"""

    email = models.EmailField(_('email address'), max_length=254, unique=True)

    class Meta:
        abstract = True

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'email'

    def clean(self):
        """Override default clean method to normalize email.

        Call :code:`super().clean()` if overriding.

        """
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Sends an email to this User."""
        send_mail(subject, message, from_email, [self.email], **kwargs)


# pylint: disable=too-many-ancestors
class AbstractUser(
        DjangoIntegrationMixin, FullNameMixin, ShortNameMixin, EmailAuthMixin,
        PermissionsMixin, AbstractBaseUser):
    """
    An abstract base class meant to be inherited (do not instantiate
    this). The class provides a fully featured User model with
    admin-compliant permissions. Differs from Django's
    :class:`~django.contrib.auth.models.AbstractUser`:

    1. Login occurs with an email and password instead of username.
    2. Provides short_name and full_name instead of first_name and
       last_name.

    All fields other than email and password are optional.

    Sets :attr:`~django.db.models.Model.objects` to
    :class:`~improved_user.models.UserManager`.

    Documentation about Django's
    :class:`~django.contrib.auth.models.AbstractBaseUser` may be helpful
    in understanding this class.
    """
    objects = UserManager()

    # misnomer; fields Dj prompts for when user calls createsuperuser
    # https://docs.djangoproject.com/en/stable/topics/auth/customizing/#django.contrib.auth.models.CustomUser.REQUIRED_FIELDS
    REQUIRED_FIELDS = ['full_name', 'short_name']

    class Meta:
        abstract = True
        verbose_name = _('user')
        verbose_name_plural = _('users')



"""Test basic functionality; test API used by a Django project developer"""
from unittest import skipUnless

from django import VERSION as DjangoVersion
from django.contrib.auth import get_user, get_user_model
from django.http import HttpRequest
from django.test import TestCase
from django.utils import translation

from improved_user.models import User


class BasicTestCase(TestCase):
    """Test Improved User to mimic Django Auth User

    The goal is to provide a User model that can be used as a drop-in
    replacement for Django Auth User, all while providing mixin classes
    to allow developers to easily extend their own class. These tests
    focus on the first part.
    """

    def test_user_creation(self):
        """Users can be created and can set/modify their password"""
        email_lowercase = 'test@example.com'
        password = 'password1!'
        user = User.objects.create_user(email_lowercase, password)
        self.assertEqual(user.email, email_lowercase)
        self.assertTrue(user.has_usable_password())
        self.assertFalse(user.check_password('wrong'))
        self.assertTrue(user.check_password(password))

        # Check we can manually set an unusable password
        user.set_unusable_password()
        user.save()
        self.assertFalse(user.check_password(password))
        self.assertFalse(user.has_usable_password())
        user.set_password(password)
        self.assertTrue(user.check_password(password))
        user.set_password(None)
        self.assertFalse(user.has_usable_password())

        # can add short and full name
        user.full_name = 'John Smith'
        user.short_name = 'John'
        user.save()
        self.assertEqual(user.get_full_name(), 'John Smith')
        self.assertEqual(user.get_short_name(), 'John')

    def test_user_creation_without_password(self):
        """Users can be created without password"""
        user = User.objects.create_user('test@example.com')
        self.assertFalse(user.has_usable_password())

    def test_unicode_email(self):  # pylint: disable=no-self-use
        """Unicode emails are allowed in Model

        Note that Django's Email field validator will error on
        the following, meaning that while the model accepts
        these values, the forms will not. Some work required.

        """
        User.objects.create_user('Pelé@example.com')
        User.objects.create_user('δοκιμή@παράδειγμα.δοκιμή')
        User.objects.create_user('我買@屋企.香港')
        User.objects.create_user('甲斐@黒川.日本')
        User.objects.create_user('чебурашка@ящик-с-апельсинами.рф')
        User.objects.create_user('संपर्क@डाटामेल.भारत')
        # Unlike usernames, emails are not normalized,
        # identical glyphs with different codepoints are allowed
        omega_emails = 'iamtheΩ@email.com'  # U+03A9 GREEK CAPITAL LETTER OMEGA
        ohm_username = 'iamtheΩ@email.com'  # U+2126 OHM SIGN
        User.objects.create_user(omega_emails)
        User.objects.create_user(ohm_username)

    def test_user_permissions(self):
        """Test normal user's authentication permissions"""
        user = User.objects.create_user('test@example.com')
        # Check authentication/permissions
        if DjangoVersion >= (1, 10):
            self.assertFalse(user.is_anonymous)
            self.assertTrue(user.is_authenticated)
        else:
            self.assertFalse(user.is_anonymous())
            self.assertTrue(user.is_authenticated())
        self.assertFalse(user.is_staff)
        self.assertTrue(user.is_active)
        self.assertFalse(user.is_superuser)

    def test_superuser_permissions(self):
        """Test superuser's authentication permissions"""
        user = User.objects.create_superuser('test@example.com', 'password1!')
        if DjangoVersion >= (1, 10):
            self.assertFalse(user.is_anonymous)
            self.assertTrue(user.is_authenticated)
        else:
            self.assertFalse(user.is_anonymous())
            self.assertTrue(user.is_authenticated())
        self.assertTrue(user.is_superuser)
        self.assertTrue(user.is_active)
        self.assertTrue(user.is_staff)

    def test_username_getter(self):
        """Check username getter method"""
        user = User.objects.create_user('test@example.com')
        self.assertEqual(user.get_username(), 'test@example.com')

    @skipUnless(
        DjangoVersion >= (1, 11),
        'Method not implemented until Django 1.11')
    def test_default_email_method(self):
        """Test correct email field used in method"""
        user = User()
        self.assertEqual(user.get_email_field_name(), 'email')

    def test_default_email_field(self):
        """Test correct email field used"""
        self.assertEqual(User.EMAIL_FIELD, 'email')

    def test_is_active(self):
        """Test that is_active can be modified"""
        user = User.objects.create(email='foo@bar.com')
        # is_active is true by default
        self.assertIs(user.is_active, True)
        user.is_active = False
        user.save()
        user_fetched = User.objects.get(pk=user.pk)
        # the is_active flag is saved
        self.assertIs(user_fetched.is_active, False)

    def test_is_staff(self):
        """Test that is_staff can be modified"""
        user = User.objects.create(email='foo@bar.com')
        # is_active is true by default
        self.assertIs(user.is_staff, False)
        user.is_staff = True
        user.save()
        user_fetched = User.objects.get(pk=user.pk)
        # the is_active flag is saved
        self.assertIs(user_fetched.is_staff, True)

    def test_is_superuser(self):
        """Test that is_superuser can be modified"""
        user = User.objects.create(email='foo@bar.com')
        # is_active is true by default
        self.assertIs(user.is_superuser, False)
        user.is_superuser = True
        user.save()
        user_fetched = User.objects.get(pk=user.pk)
        # the is_active flag is saved
        self.assertIs(user_fetched.is_superuser, True)

    def test_get_user_model(self):
        """The improved user model can be retrieved"""
        self.assertEqual(get_user_model(), User)
        with self.assertRaises(AttributeError):
            from django.contrib.auth.models import User as DjangoUser
            DjangoUser.objects.all()

    def test_user_verbose_names_translatable(self):
        """User model verbose names are translatable (#19945)"""
        with translation.override('en'):
            self.assertEqual(User._meta.verbose_name, 'user')
            self.assertEqual(User._meta.verbose_name_plural, 'users')
        with translation.override('es'):
            self.assertEqual(User._meta.verbose_name, 'usuario')
            self.assertEqual(User._meta.verbose_name_plural, 'usuarios')

    def test_get_user(self):
        """Improved User can be extracted from request"""
        created_user = User.objects.create_user('test@example.com', 'testpw')
        self.client.login(username='test@example.com', password='testpw')
        request = HttpRequest()
        request.session = self.client.session
        user = get_user(request)
        self.assertIsInstance(user, User)
        self.assertEqual(user.email, created_user.email)



"""
Django Improved User Library

1. define directory as module
2. set default app config

https://docs.djangoproject.com/en/stable/ref/applications/#configuring-applications

"""
# pylint: disable=invalid-name
default_app_config = 'improved_user.apps.ImprovedUserConfig'
# pylint: enable=invalid-name



#!/usr/bin/env python3
"""
====================
Django Improved User
====================

:website: https://github.com/jambonsw/django-improved-user/
:copyright: Copyright 2017 JamBon Software
:license: Simplified BSD, see LICENSE for details.
"""

from distutils.command.check import check as CheckCommand
from operator import attrgetter
from os.path import abspath, dirname, join

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand

HERE = abspath(dirname(__file__))


def load_file_contents(file_path, as_list=True):
    """Load file as string or list"""
    abs_file_path = join(HERE, file_path)
    with open(abs_file_path, encoding='utf-8') as file_pointer:
        if as_list:
            return file_pointer.read().splitlines()
        return file_pointer.read()


LONG_DESCRIPTION = (
    load_file_contents('README.rst', as_list=False)
    .split('.. end-badges')[1]  # remove badge icons at top
    .lstrip()  # remove any extraneous spaces before title
)


class CustomCheckCommand(CheckCommand):
    """Customized distutils check command"""
    # https://github.com/python/cpython/blob/master/Lib/distutils/command/check.py
    user_options = CheckCommand.user_options + [
        ('disable-metadata', None, "don't check meta-data"),
        ('enforce-email', 'e', 'Ensure that all author/maintainer use email'),
    ]
    negative_opt = {'disable-metadata': 'metadata'}

    def initialize_options(self):
        """Setup superclass and new options"""
        super().initialize_options()
        self.enforce_email = 0  # pylint:disable=attribute-defined-outside-init

    def check_metadata(self):
        """
        Ensures that all required elements of meta-data are supplied.

        Specifically: name, version, URL, author or maintainer
        Warns if any are missing.

        If enforce-email option is true, author and/or maintainer must
        specify an email.

        """
        metadata = self.distribution.metadata

        missing = []
        for attr in ('name', 'version', 'url'):
            if not (hasattr(metadata, attr) and getattr(metadata, attr)):
                missing.append(attr)

        # https://www.python.org/dev/peps/pep-0345/
        # author or maintainer must be specified
        # author is preferred; if identifcal, specify only author
        if not metadata.author and not metadata.maintainer:
            missing.append('author')
            if self.enforce_email:
                missing.append('author_email')
        else:
            # one or both of author or maintainer specified
            if (metadata.author and self.enforce_email
                    and not metadata.author_email):
                missing.append('author_email')
            if (metadata.maintainer and self.enforce_email
                    and not metadata.maintainer_email):
                missing.append('maintainer_email')
            if (metadata.author and metadata.maintainer
                    and metadata.author == metadata.maintainer):
                self.warn(
                    'Maintainer should be omitted if identical to Author.\n'
                    'See https://www.python.org/dev/peps/pep-0345/'
                    '#maintainer-email-optional')
            if (metadata.author_email and metadata.maintainer_email
                    and metadata.author_email == metadata.maintainer_email):
                self.warn(
                    'Maintainer Email should be omitted if'
                    "identical to Author's.\n"
                    'See https://www.python.org/dev/peps/pep-0345/'
                    '#maintainer-email-optional')

        if missing:
            self.warn('missing required meta-data: %s' % ', '.join(missing))


class CustomTestCommand(TestCommand):
    """Customized setuptools test command"""
    # https://github.com/pypa/setuptools/blob/master/setuptools/command/test.py
    # https://github.com/python/cpython/blob/master/Lib/distutils/cmd.py
    description = 'run project tests'
    command_consumes_arguments = True
    user_options = []

    def initialize_options(self):
        """Explicitly set all instance vars"""
        self.args = []  # pylint: disable=attribute-defined-outside-init

    def finalize_options(self):
        """Needed by Superclass"""
        pass

    def run(self):
        installed_dists = self.install_dists(self.distribution)

        if self.dry_run:
            self.announce('skipping tests (dry run)')
            return

        paths = map(attrgetter('location'), installed_dists)
        with self.paths_on_pythonpath(paths):
            with self.project_on_sys_path():
                self.run_tests()

    def run_tests(self):
        from runtests import (
            check_missing_migrations, configure_django, run_test_suite,
        )
        configure_django()
        check_missing_migrations()
        run_test_suite(*self.args)


setup(
    name='django-improved-user',
    version='1.0a2',
    description=(
        'A custom Django user model for best practices email-based login.'
    ),
    long_description=LONG_DESCRIPTION,
    url='https://github.com/jambonsw/django-improved-user/',

    packages=find_packages('src'),
    package_dir={'': 'src'},
    install_requires=[
        'django>=1.8,!=1.9.*',
    ],
    extras_require={
        'factory': [
            'factory_boy==2.9.2',
            'Faker==0.8.5',
            'python-dateutil==2.6.1',
        ],
    },
    zip_safe=False,

    cmdclass={
        'check': CustomCheckCommand,
        'test': CustomTestCommand,
    },

    author=(
        'Russell Keith-Magee, '
        'Andrew Pinkham'
    ),
    license='BSD License',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3 :: Only',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
    ],
)



"""User Manager used by Improved User; may be extended"""
from django.contrib.auth.models import BaseUserManager


class UserManager(BaseUserManager):
    """Manager for Users; overrides create commands for new fields

    Meant to be interacted with via the user model.

    .. code:: python

        User.objects  # the UserManager
        User.objects.all()  # has normal Manager/UserManager methods
        User.objects.create_user  # overrides methods for Improved User

    Set to :attr:`~django.db.models.Model.objects` by
    :attr:`~improved_user.models.AbstractUser`
    """

    def _create_user(
            self, email, password, is_staff, is_superuser, **extra_fields):
        """Helper method to save a User with improved user fields"""
        if not email:
            raise ValueError('An email address must be provided.')
        if 'username' in extra_fields:
            raise ValueError(
                'The Improved User model does not have a username; '
                'it uses only email')
        user = self.model(
            email=self.normalize_email(email),
            is_staff=is_staff, is_superuser=is_superuser,
            **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, email=None, password=None, **extra_fields):
        """Save new User with email and password"""
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        return self._create_user(email, password, **extra_fields)

    def create_superuser(self, email, password, **extra_fields):
        """Save new User with is_staff and is_superuser set to True"""
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')
        return self._create_user(email, password, **extra_fields)



"""Test Improved User Model"""
from datetime import datetime
from types import MethodType
from unittest import skipUnless
from unittest.mock import patch

from django import VERSION as DjangoVersion
from django.contrib.auth.hashers import get_hasher
from django.core import mail
from django.test import TestCase

from improved_user.models import User


class UserModelTestCase(TestCase):
    """Improve User Model Test Suite"""

    def test_fields_and_attributes(self):
        """Ensure the model has the fields and attributes we expect"""
        expected_fields = (
            'id',
            'password',
            'last_login',
            'is_superuser',
            'full_name',
            'short_name',
            'is_staff',
            'is_active',
            'date_joined',
            'email',
            'groups',
            'user_permissions',
        )
        user_fields = [field.name for field in User._meta.get_fields()]
        for field in expected_fields:
            with self.subTest(field=field):
                self.assertIn(field, user_fields)
        # Pre-empt Django check auth.E001
        self.assertTrue(isinstance(User.REQUIRED_FIELDS, (list, tuple)))
        # Pre-empt Django check auth.E002
        self.assertNotIn(User.USERNAME_FIELD, User.REQUIRED_FIELDS)
        # Pre-empt Django check auth.E003
        self.assertIs(User._meta.get_field(User.USERNAME_FIELD).unique, True)
        # Pre-empt Django check auth.C009
        self.assertFalse(isinstance(User.is_anonymous, MethodType))
        # Pre-empt Django check auth.C010
        self.assertFalse(isinstance(User.is_authenticated, MethodType))

    def test_email_user(self):
        """Send Email to User via method"""
        # valid send_mail parameters
        kwargs = {
            'fail_silently': False,
            'auth_user': None,
            'auth_password': None,
            'connection': None,
            'html_message': None,
        }
        user = User(email='foo@bar.com')
        user.email_user(
            subject='Subject here',
            message='This is a message',
            from_email='from@domain.com',
            # TODO: when Py3.4 removed, add comma, remove C815 exception
            **kwargs  # noqa: C815
        )
        self.assertEqual(len(mail.outbox), 1)
        message = mail.outbox[0]
        self.assertEqual(message.subject, 'Subject here')
        self.assertEqual(message.body, 'This is a message')
        self.assertEqual(message.from_email, 'from@domain.com')
        self.assertEqual(message.to, [user.email])

    def test_last_login_default(self):
        """Check last login not set upon creation"""
        user1 = User.objects.create(email='test1@example.com')
        self.assertIsNone(user1.last_login)

        user2 = User.objects.create(email='test2@example.com')
        self.assertIsNone(user2.last_login)

    def test_date_joined_default(self):
        """Check date joined set upon creation"""
        user1 = User.objects.create(email='test1@example.com')
        self.assertIsNotNone(user1.date_joined)
        self.assertIsInstance(user1.date_joined, datetime)

        user2 = User.objects.create(email='test2@example.com')
        self.assertIsNotNone(user2.date_joined)
        self.assertIsInstance(user2.date_joined, datetime)

    def test_user_clean_normalize_email(self):
        """User email/username is normalized upon creation"""
        user = User(email='foo@BAR.com', password='foo')
        user.clean()
        self.assertEqual(user.email, 'foo@bar.com')

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    def test_user_double_save(self):
        """
        Calling user.save() twice should trigger password_changed() once.
        """
        user = User.objects.create_user(
            email='test@example.com', password='foo')
        user.set_password('bar')
        with patch(
            'django.contrib.auth.password_validation.password_changed',
        ) as pw_changed:
            user.save()
            self.assertEqual(pw_changed.call_count, 1)
            user.save()
            self.assertEqual(pw_changed.call_count, 1)

    @skipUnless(
        DjangoVersion >= (1, 9),
        'Password strength checks not available on Django 1.8')
    def test_check_password_upgrade(self):
        """
        password_changed() shouldn't be called if User.check_password()
        triggers a hash iteration upgrade.
        """
        user = User.objects.create_user(
            email='test@example.com', password='foo')
        initial_password = user.password
        self.assertTrue(user.check_password('foo'))
        hasher = get_hasher('default')
        self.assertEqual('pbkdf2_sha256', hasher.algorithm)

        old_iterations = hasher.iterations
        try:
            # Upgrade the password iterations
            hasher.iterations = old_iterations + 1
            with patch(
                'django.contrib.auth.password_validation.password_changed',
            ) as pw_changed:
                user.check_password('foo')
                self.assertEqual(pw_changed.call_count, 0)
            self.assertNotEqual(initial_password, user.password)
        finally:
            hasher.iterations = old_iterations
